/********************************************************************************
*                                                                               *
*             B a c k  B u f f e r e d  C a n v a s  W i d g e t                *
*                                                                               *
*********************************************************************************
* Copyright (C) 2003 by Davy Durham.  With coding tips from Jeroen van der Zijp *
* Copyright (C) 2003 by Dimitris Servis.  Hacked from Davy Durham's code        *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
*********************************************************************************
* Special thanks to Jeroen van der Zijp for tips and tricks                     *
********************************************************************************/

#include "FXBackBufferedCanvas.h"
/// NOTE : TIFF, JPEG, and  PNG image formats may need additional libraries and
/// headers (at least for VC++. Dev - C++ already has some.
/// You may find them under the download page of the FOX site (last items).

#ifdef HAVE_TIFF_H
#include "FXTIFImage.h"
#endif

#ifdef HAVE_JPEG_H
#include "FXJPGImage.h"
#endif

#ifdef HAVE_PNG_H
#include "FXPNGImage.h"
#endif


// Map
FXDEFMAP(FXBackBufferedCanvas) FXBackBufferedCanvasMap[]= {
  FXMAPFUNC(SEL_PAINT,0,FXBackBufferedCanvas::onPaint),
  FXMAPFUNC(SEL_CONFIGURE,0,FXBackBufferedCanvas::onConfigure),
  };
FXIMPLEMENT(FXBackBufferedCanvas,FXCanvas,FXBackBufferedCanvasMap,ARRAYNUMBER(FXBackBufferedCanvasMap))


// serialisation
FXBackBufferedCanvas::FXBackBufferedCanvas() : FXCanvas() {
  backBuffer=new FXImage(getApp(),NULL,0,0,0);
  currentDC=NULL;
  }


// Construct new back buffered drawing canvas widget
FXBackBufferedCanvas::FXBackBufferedCanvas(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : 
  FXCanvas(p,tgt,sel,opts,x,y,w,h) {
  backBuffer=new FXImage(getApp(),NULL,0,w,h);
  currentDC=NULL;
  }

// cleanup
FXBackBufferedCanvas::~FXBackBufferedCanvas() {
  delete backBuffer;
  delete currentDC;
  }

// create resources
void FXBackBufferedCanvas::create() {
  FXCanvas::create();
  backBuffer->create();
   currentDC=NULL;
 }

// save resources
void FXBackBufferedCanvas::save(FXStream& store) const {
  FXCanvas::save(store);
  store << backBuffer;
  }

// load resources
void FXBackBufferedCanvas::load(FXStream& store) {
  FXCanvas::load(store);
  store >> backBuffer;
  }

// NOTE: if IMAGE_OWNED is in the flags, then there is really no need to allocate an FXDCWindow
//     and return it via the ptr parameter in the onPaint even or via the return value of
//     beginPaint()
void FXBackBufferedCanvas::setBackBufferOptions(FXint opt) {
/// Destroy the back buffer in order to change options
  if(backBuffer)
  {
	  delete backBuffer;
	  backBuffer=(FXImage*)-1;
  }
/// create the new backbuffer but keep the pixmap
  if(opt&IMAGE_OWNED) opt|=IMAGE_KEEP;
  backBuffer=new FXImage(getApp(),NULL,opt,getWidth(),getHeight());
  backBuffer->create();
  }

// return handle to back-buffer data
void* FXBackBufferedCanvas::getBackBufferData() {
	return backBuffer?backBuffer->getData():NULL;
  }

// Handle paint event.  If synthetic event, allow target a chance at drawing into backbuffer
// before blitting it to screen
//
// Note that we dont handle the scenario where someone has called beginPaint(), then this event
// is called - it is a programming error. ie the user is not allowed to call beginPaint and
// endPaint within the onPaint handler...
long FXBackBufferedCanvas::onPaint(FXObject*,FXSelector,void *ptr) {
  if (currentDC) fxerror("%s: multiple trigger of onPaint.\n",getClassName());
  FXEvent *ev=(FXEvent*)ptr;

  // triggered by a call to update()
	if(ev->synthetic && target) 
	{
		currentDC=new FXDCWindow(backBuffer);
		currentDC->setForeground(getBackColor());
		currentDC->fillRectangle(0,0,backBuffer->getWidth(),backBuffer->getHeight());
		target->handle(this,FXSEL(SEL_PAINT,message),ptr);
  }

  // now blit the back buffer to the screen
  FXDCWindow screenDC(this,ev); 
  // in case we've changed the options to keep the image, we do not need to render
  if(backBuffer->getOptions()&~(IMAGE_OWNED|IMAGE_KEEP)) backBuffer->render();
  screenDC.drawImage(backBuffer,0,0);
  return 1;
  }

// Resized .. so resize the back buffer and repaint it
long FXBackBufferedCanvas::onConfigure(FXObject *sender,FXSelector sel,void *ptr) {
  if(target) target->handle(this,FXSEL(SEL_CONFIGURE,message),ptr);
  backBuffer->resize(getWidth(),getHeight());
  update();
  return 1;
  }

// return handle to current back buffer, so that you can draw into it
FXDCWindow* FXBackBufferedCanvas::beginPaint() {
  if(currentDC) return currentDC;
  currentDC=new FXDCWindow(backBuffer);
  currentDC->setForeground(getBackColor());
  currentDC->fillRectangle(0,0,backBuffer->getWidth(),backBuffer->getHeight());

  return currentDC;
}

// finish drawing to backbuffer - blits buffer to screen
void FXBackBufferedCanvas::endPaint() {
  if(currentDC) {
  currentDC->end();
  delete currentDC;
  currentDC=(FXDCWindow*)-1;
  currentDC=NULL;

  }
}

/// Assign and create new hidden image to backbuffer and invoke a
/// SEL_PAINT event so that the target may draw on the new image
/// Note, in this case, the image is offscreen
void FXBackBufferedCanvas::createHiddenImage(FXImage* newimage) {
  if(backBuffer){
   delete backBuffer;
   backBuffer=(FXImage*)-1L;
  }
  backBuffer=newimage;
  backBuffer->create();

  if(target) target->handle(this,FXSEL(SEL_PAINT,message),NULL);
  backBuffer->restore();
}

/// Destroy the hidden image after saving it
void FXBackBufferedCanvas::destroyHiddenImage() {
  delete backBuffer;
  backBuffer=(FXImage*)-1L;
  backBuffer=new FXImage(getApp(),NULL,0,getWidth(),getHeight());
  backBuffer->create();
}
//// Save an image
///  These two methods save an image of a predefined type.
///  It is assumed that the filename contains the appropriate extension 
FXint FXBackBufferedCanvas::saveImage(FXchar* file, FXuint type, FXuint w, FXuint h) {
  FXString filename(file);
  return saveImage(filename,type,w,h);
}

FXint FXBackBufferedCanvas::saveImage(FXString& filename, FXuint type, FXuint w, FXuint h) {
///  In case no width or height is defined, use the width and height of the backbuffer
  if (!w) w=backBuffer->getWidth();
  if (!h) h=backBuffer->getHeight();
/// Setup the options for the hidden image
  FXuint opt=IMAGE_OWNED|IMAGE_KEEP;
/// the return value is the image type
  FXint err=type;
/// set up a filestream
  FXFileStream afile;
/// try to open it with the given filename
  if(!afile.open(filename.text(),FXStreamSave))
  {
#ifdef _DEBUG
    fxwarning("FXBackBufferedCanvas::saveImage: could not open file");
#endif
    return 0;
  }

/// resolve for the appropriate image type, and create the relevant image
  switch(type)
  {
  case SAVE_BMP:
       createHiddenImage(new FXBMPImage(getApp(),NULL,opt,w,h));
       break;
  case SAVE_GIF:
       createHiddenImage(new FXGIFImage(getApp(),NULL,opt,w,h));
       break;
  case SAVE_TGA:
       createHiddenImage(new FXTGAImage(getApp(),NULL,opt,w,h));
       break;
  case SAVE_XPM:
       createHiddenImage(new FXXPMImage(getApp(),NULL,opt,w,h));
       break;
  case SAVE_PCX:
       createHiddenImage(new FXPCXImage(getApp(),NULL,opt,w,h));
       break;
  case SAVE_TIF:
#ifdef HAVE_TIFF_H
       createHiddenImage(new FXTIFImage(getApp(),NULL,opt,w,h));
#endif
       break;
	case SAVE_JPG:
#ifdef HAVE_JPEG_H
       createHiddenImage(new FXJPGImage(getApp(),NULL,opt,w,h));
#endif
       break;
	case SAVE_PNG:
#ifdef HAVE_PNG_H
       createHiddenImage(new FXPNGImage(getApp(),NULL,opt,w,h));
#endif
       break;
  }

/// Try to save the image to the file
  if(!backBuffer->savePixels(afile)) 
  {
#ifdef _DEBUG
    fxwarning("FXBackBufferedCanvas::saveImage: could not open file");
#endif
    err=0;
  }
/// destroy the image no matter what and close the file
  destroyHiddenImage();
  afile.close();
/// return 0 if unsuccesful, the image type saved if succesful
  return err;
}

