/********************************************************************************
*                                                                               *
*             B a c k  B u f f e r e d  C a n v a s  W i d g e t                *
*                                                                               *
*********************************************************************************
* Copyright (C) 2003 by Davy Durham.  With coding tips from Jeroen van der Zijp *
* Copyright (C) 2003 by Dimitris Servis.  Hacked from Davy Durham's code        *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
*********************************************************************************
* Special thanks to Jeroen van der Zijp for tips and tricks                     *
********************************************************************************/
#ifndef FXBACKBUFFEREDCANVAS_H
#define FXBACKBUFFEREDCANVAS_H


#include "fx.h"

using namespace FX ;

enum FXBBImageFormats {
  SAVE_BMP=1,     /// Save BMP etc.
  SAVE_GIF, 
  SAVE_TGA,
  SAVE_XPM,
  SAVE_PCX,
  SAVE_TIF,
  SAVE_JPG,
  SAVE_PNG
};

/**
* Back Buffered Canvas, an area drawn by another object, except a little more
* expensive but to reduce flicker.
* Enhanced to output sized image formats supported by FOX
*/

class FXAPI FXBackBufferedCanvas : public FXCanvas {
  FXDECLARE(FXBackBufferedCanvas)

protected:
  FXImage*		backBuffer;     // the off-screen buffer
  FXDCWindow*	currentDC;      // handle to current FXDC

protected:
  /// serialisation
  FXBackBufferedCanvas();
public:
  long onPaint(FXObject*,FXSelector,void*);
  long onConfigure(FXObject*,FXSelector,void*);
protected:
  /// create a hidden image for off screen drawing in order to export it
  void createHiddenImage(FXImage*);
  /// destroy the hidden image 
  void destroyHiddenImage();
public:
  /// Construct new back buffered drawing canvas widget
  FXBackBufferedCanvas(FXComposite* p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=FRAME_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  /// create resources
  void create();

  /*
   * - This can be used to set the options of the back buffer to include
   * IMAGE_OWNED so that the drawing code can call getBackBufferData()
   * and render/modify the back buffer.
   * - When IMAGE_OWNED is included in the options, then backBuffer->render()
   * will be called just before the backbuffer is blitted to the screen.
   */
  void setBackBufferOptions(FXint);

  /// return handle to some backbuffered data
  void *getBackBufferData();

  /**
   * Normally, the user should wait for the SEL_PAINT event to draw the the canvas, in which case
   * the ptr parameter would contain the DC to draw to. 
   *
   * However, these two methods need to be used if the user is not waiting for the SEL_PAINT event 
   * to draw to the canvas since the user should actually be drawing to the back buffer and not the
   * actual canvas on screen.  (I suppose much more work could be done to make the use of back
   * buffering completly transparent and make it have the same interface as FXCanvas)
   */
  FXDCWindow *beginPaint();

  /// finish painting
  void endPaint();

  /// Save object to stream
  virtual void save(FXStream& store) const;

  /// Load object from stream
  virtual void load(FXStream& store);

  /// dtor
  virtual ~FXBackBufferedCanvas();
  /// get current dc to draw
  FXDCWindow* getCurrentDC() const { return currentDC; }
  /// Save an image in one of the supported formats
  FXint saveImage( FXString& filename, FXuint type=SAVE_BMP, FXuint w=0, FXuint h=0);
  FXint saveImage( FXchar* file, FXuint type=SAVE_BMP, FXuint w=0, FXuint h=0);


  };

#endif // FXBACKBUFFEREDCANVAS_H
